#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.01.23 19:00:00                  #
# ================================================== #

import datetime
import os

from PySide6.QtGui import QImage

from pygpt_net.utils import trans


class Capture:
    def __init__(self, window=None):
        """
        Painter capture controller

        :param window: Window instance
        """
        self.window = window

    def camera(self):
        """Get image from camera"""
        if not self.window.controller.camera.is_enabled():
            self.window.controller.camera.enable_capture()
            self.window.controller.camera.setup_ui()
        frame = self.window.controller.camera.get_current_frame(False)
        if frame is None:
            return
        height, width, channel = frame.shape
        bytes = 3 * width
        image = QImage(frame.data, width, height, bytes, QImage.Format_RGB888)
        self.window.ui.painter.set_image(image)

    def screenshot(self):
        """Make screenshot and append to attachments"""
        # switch to vision mode if needed
        self.window.controller.chat.vision.switch_to_vision()

        # clear attachments before capture if needed
        if self.window.controller.attachment.is_capture_clear():
            self.window.controller.attachment.clear(True)

        try:
            # prepare filename
            now = datetime.datetime.now()
            dt = now.strftime("%Y-%m-%d_%H-%M-%S")
            name = 'cap-' + dt
            path = os.path.join(self.window.controller.painter.common.get_capture_dir(), name + '.png')

            # capture screenshot
            screen = self.window.app.primaryScreen()
            screenshot = screen.grabWindow(0)
            screenshot.save(path, 'png')
            self.attach(name, path, 'screenshot')
            self.window.controller.painter.open(path)

            # show last capture time in status
            dt_info = now.strftime("%Y-%m-%d %H:%M:%S")
            self.window.statusChanged.emit(trans("painter.capture.manual.captured.success") + ' ' + dt_info)
            return True

        except Exception as e:
            print("Screenshot capture exception", e)
            self.window.core.debug.log(e)

    def use(self):
        """Use current image"""
        # switch to vision mode if needed
        self.window.controller.chat.vision.switch_to_vision()

        # clear attachments before capture if needed
        if self.window.controller.attachment.is_capture_clear():
            self.window.controller.attachment.clear(True)

        try:
            # prepare filename
            now = datetime.datetime.now()
            dt = now.strftime("%Y-%m-%d_%H-%M-%S")
            name = 'cap-' + dt
            path = os.path.join(self.window.controller.painter.common.get_capture_dir(), name + '.png')

            # capture
            self.window.ui.painter.image.save(path)
            self.attach(name, path)

            # show last capture time in status
            dt_info = now.strftime("%Y-%m-%d %H:%M:%S")
            self.window.statusChanged.emit(trans("painter.capture.manual.captured.success") + ' ' + dt_info)
            return True

        except Exception as e:
            print("Image capture exception", e)
            self.window.core.debug.log(e)

    def attach(self, name: str, path: str, type: str = 'drawing'):
        """
        Attach image to attachments

        :param name: image name
        :param path: image path
        :param type: capture type (drawing, screenshot)
        """
        mode = self.window.core.config.get('mode')
        if type == 'drawing':
            title = trans('painter.capture.name.prefix') + ' ' + name
        elif type == 'screenshot':
            title = trans('screenshot.capture.name.prefix') + ' ' + name
        else:
            title = name
        title = title.replace('cap-', '').replace('_', ' ')

        # make attachment
        self.window.core.attachments.new(mode, title, path, False)
        self.window.core.attachments.save()
        self.window.controller.attachment.update()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.01.30 13:00:00                  #
# ================================================== #

import serial
import time
from PySide6.QtCore import Slot

from pygpt_net.plugin.base import BaseWorker, BaseSignals


class WorkerSignals(BaseSignals):
    pass  # add custom signals here


class Worker(BaseWorker):
    def __init__(self, *args, **kwargs):
        super(Worker, self).__init__()
        self.signals = WorkerSignals()
        self.args = args
        self.kwargs = kwargs
        self.plugin = None
        self.cmds = None
        self.ctx = None

    @Slot()
    def run(self):
        msg = None
        for item in self.cmds:
            try:
                if item["cmd"] in self.plugin.allowed_cmds and self.plugin.has_cmd(item["cmd"]):
                    request = {"cmd": item["cmd"]}  # prepare request item for result

                    # serial: send text command
                    if item["cmd"] == "serial_send":
                        port = self.plugin.get_option_value("serial_port")
                        speed = self.plugin.get_option_value("serial_bps")
                        timeout = self.plugin.get_option_value("timeout")
                        sleep = self.plugin.get_option_value("sleep")
                        self.log("Using serial port: {} @ {} bps".format(port, speed))
                        try:
                            msg = "Sending command to USB port: {}".format(
                                item["params"]['command'],
                            )
                            self.log(msg)
                            data = self.send_command(
                                port,
                                speed,
                                item["params"]['command'],
                                timeout=timeout,
                                sleep=sleep,
                            )
                            self.log("Response: {}".format(data))
                            response = {
                                "request": request,
                                "result": data,
                            }
                        except Exception as e:
                            response = {
                                "request": request,
                                "result": "Error: {}".format(e),
                            }
                            self.error(e)
                            self.log("Error: {}".format(e))
                        self.response(response)

                    # serial: send raw bytes command
                    elif item["cmd"] == "serial_send_bytes":
                        port = self.plugin.get_option_value("serial_port")
                        speed = self.plugin.get_option_value("serial_bps")
                        timeout = self.plugin.get_option_value("timeout")
                        sleep = self.plugin.get_option_value("sleep")
                        self.log("Using serial port: {} @ {} bps".format(port, speed))
                        try:
                            msg = "Sending binary data to USB port: {}".format(
                                item["params"]['bytes'],
                            )
                            self.log(msg)
                            data = self.send_binary_data(
                                port,
                                speed,
                                int(item["params"]['bytes']),
                                timeout=timeout,
                                sleep=sleep,
                            )
                            self.log("Response: {}".format(data))
                            response = {
                                "request": request,
                                "result": data,
                            }
                        except Exception as e:
                            response = {
                                "request": request,
                                "result": "Error: {}".format(e),
                            }
                            self.error(e)
                            self.log("Error: {}".format(e))
                        self.response(response)

                    # serial: read data from USB port
                    elif item["cmd"] == "serial_read":
                        port = self.plugin.get_option_value("serial_port")
                        speed = self.plugin.get_option_value("serial_bps")
                        timeout = self.plugin.get_option_value("timeout")
                        duration = int(item["params"]['duration']) \
                            if "duration" in item["params"] else 3
                        self.log("Using serial port: {} @ {} bps".format(port, speed))
                        try:
                            msg = "Reading data from USB port..."
                            self.log(msg)
                            data = self.read_data(
                                port,
                                speed,
                                timeout=timeout,
                                duration=duration,
                            )
                            self.log("Response: {}".format(data))
                            response = {
                                "request": request,
                                "result": data,
                            }
                        except Exception as e:
                            response = {
                                "request": request,
                                "result": "Error: {}".format(e),
                            }
                            self.error(e)
                            self.log("Error: {}".format(e))
                        self.response(response)

            except Exception as e:
                self.response(
                    {
                        "request": item,
                        "result": "Error: {}".format(e),
                    }
                )
                self.error(e)
                self.log("Error: {}".format(e))

        if msg is not None:
            self.status(msg)

    def send_command(
            self,
            port: str,
            speed: int,
            command: str,
            timeout: int = 1,
            sleep: int = 2
    ) -> str:
        """
        Send command to USB port

        :param port: USB port name, e.g. /dev/ttyACM0
        :param speed: Port connection speed, in bps, default: 9600
        :param command: Command to send
        :param timeout: Timeout in seconds
        :param sleep: Sleep time in seconds
        :return: Response from USB port
        """
        ser = serial.Serial(port, speed, timeout=timeout)
        time.sleep(sleep)
        ser.write((command + '\n').encode())
        ser.flush()
        return ser.readline().decode().strip()

    def send_binary_data(
            self,
            port: str,
            speed: int,
            data: int,
            timeout: int = 1,
            sleep: int = 2
    ) -> str:
        """
        Send command to USB port

        :param port: USB port name, e.g. /dev/ttyACM0
        :param speed: Port connection speed, in bps, default: 9600
        :param data: Data to send
        :param timeout: Timeout in seconds
        :param sleep: Sleep time in seconds
        :return: Response from USB port
        """
        ser = serial.Serial(port, speed, timeout=timeout)
        time.sleep(sleep)
        ser.write(bytes(data))
        ser.flush()
        return ser.readline().decode().strip()

    def read_data(
            self,
            port: str,
            speed: int,
            timeout: int = 1,
            duration: int = 3
    ) -> str:
        """
        Read data from USB port

        :param port: USB port name, e.g. /dev/ttyACM0
        :param speed: Port connection speed, in bps, default: 9600
        :param timeout: Timeout in seconds
        :param duration: Duration in seconds
        :return: Response from USB port
        """
        data = ""
        ser = serial.Serial(port, speed, timeout=timeout)
        end_time = time.time() + duration
        while time.time() < end_time:
            if ser.in_waiting > 0:
                data += ser.readline().decode().strip()
        return data



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.03.06 02:00:00                  #
# ================================================== #

from PySide6.QtWidgets import QWidget, QCheckBox, QHBoxLayout, QLabel
from PySide6.QtGui import QPainter, QPixmap
from PySide6.QtCore import Qt

from pygpt_net.utils import trans


class ColorCheckbox(QWidget):
    def __init__(self, window=None):
        super().__init__()
        self.window = window
        self.selected = []
        self.boxes = {}
        self.build()

    def build(self):
        """Build color labels checkboxes"""
        self.window.ui.nodes['filter.ctx.label.colors'] = QLabel(trans("filter.ctx.label.colors"))
        layout = QHBoxLayout()
        layout.addWidget(self.window.ui.nodes['filter.ctx.label.colors'])
        layout.setContentsMargins(0, 0, 0, 0)
        labels = self.window.controller.ui.get_colors()

        for id, status in labels.items():
            self.boxes[id] = QCheckBox()
            self.boxes[id].stateChanged.connect(
                lambda state, idx=id: self.update_colors(state, idx)
            )
            pixmap = QPixmap(20, 20)
            pixmap.fill(Qt.transparent)
            painter = QPainter(pixmap)
            painter.setBrush(status['color'])
            painter.drawRect(0, 0, 20, 20)
            painter.end()

            self.boxes[id].setIcon(pixmap)
            layout.addWidget(self.boxes[id])

        layout.addStretch()
        self.setLayout(layout)

    def update_colors(self, state: int, idx: int):
        """
        Update selected colors list

        :param state: checkbox state
        :param idx: color index
        """
        if state == 2:
            if idx not in self.selected:
                self.selected.append(idx)
        else:
            if idx in self.selected:
                self.selected.remove(idx)

        self.window.controller.ctx.label_filters_changed(self.selected)

    def restore(self, selected: dict):
        """
        Restore selected colors

        :param selected: selected colors
        """
        self.selected = selected
        for idx in selected:
            self.boxes[idx].setChecked(True)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.03.09 10:00:00                  #
# ================================================== #

from PySide6.QtWidgets import QApplication

from pygpt_net.utils import trans


class Extra:
    def __init__(self, window=None):
        """
        Extra actions controller

        :param window: Window instance
        """
        self.window = window
        self.audio_play_id = None

    def delete_item(self, item_id: int):
        """
        Delete ctx item

        :param item_id: Item id
        """
        self.window.controller.ctx.delete_item(item_id)

    def copy_item(self, item_id: int):
        """
        Copy ctx item to clipboard

        :param item_id: Item id
        """
        item = self.window.core.ctx.get_item_by_id(item_id)
        if item is not None:
            QApplication.clipboard().setText(item.output)
            self.window.ui.status(trans("clipboard.copied"))

    def edit_item(self, item_id: int):
        """
        Edit ctx item

        :param item_id: Item id
        """
        item = self.window.core.ctx.get_item_by_id(item_id)
        if item is not None:
            input_text = item.input
            meta_id = item.meta_id

            # store edit id
            self.window.controller.ctx.edit_meta_id = meta_id
            self.window.controller.ctx.edit_item_id = item_id

            # send to input
            self.window.controller.chat.common.clear_input()
            self.window.controller.chat.common.append_to_input(input_text)

            # show update icons (hide send, show update i cancel)
            self.edit_show()

    def edit_show(self):
        """Show edit buttons"""
        self.window.ui.nodes['input.send_btn'].setVisible(False)
        self.window.ui.nodes['input.update_btn'].setVisible(True)
        self.window.ui.nodes['input.cancel_btn'].setVisible(True)

    def edit_hide(self):
        """Hide edit buttons"""
        self.window.ui.nodes['input.send_btn'].setVisible(True)
        self.window.ui.nodes['input.update_btn'].setVisible(False)
        self.window.ui.nodes['input.cancel_btn'].setVisible(False)

    def edit_submit(self):
        """Submit edit"""
        item_id = self.window.controller.ctx.edit_item_id
        meta_id = self.window.controller.ctx.edit_meta_id
        current_meta_id = self.window.core.ctx.current
        if meta_id != current_meta_id:
            self.window.controller.ctx.select(meta_id)
        item = self.window.core.ctx.get_item_by_id(item_id)
        if item is not None:
            meta_id = item.meta_id
            self.window.core.ctx.remove_items_from(meta_id, item_id)
            self.window.controller.ctx.refresh()
            self.window.controller.ctx.edit_item_id = None
            self.window.controller.ctx.edit_meta_id = None
            self.window.controller.chat.input.send_input(force=True)
        self.edit_hide()

    def is_editing(self) -> bool:
        """
        Check if editing ctx item is in progress

        :return: True if editing
        """
        return self.window.controller.ctx.edit_item_id is not None

    def edit_cancel(self):
        """Cancel edit"""
        self.window.controller.chat.common.clear_input()
        self.window.controller.ctx.edit_item_id = None
        self.window.controller.ctx.edit_meta_id = None
        self.edit_hide()

    def replay_item(self, item_id: int, force: bool = False):
        """
        Regenerate ctx item

        :param item_id: Item id
        :param force: Force regenerate
        """
        if not force:
            self.window.ui.dialogs.confirm(
                type='ctx.replay_item',
                id=item_id,
                msg=trans('ctx.replay.item.confirm'),
            )
            return

        item = self.window.core.ctx.get_item_by_id(item_id)
        if item is not None:
            input_text = item.input
            meta_id = item.meta_id
            self.window.core.ctx.remove_items_from(meta_id, item_id)
            self.window.controller.ctx.refresh()
            self.window.controller.chat.input.send(input_text)

    def audio_read_item(self, item_id: int):
        """
        Read ctx item audio

        :param item_id: Item id
        """
        item = self.window.core.ctx.get_item_by_id(item_id)
        if item is not None:
            if self.audio_play_id is None or self.audio_play_id != item_id:
                self.window.controller.audio.stop_output()
                self.window.controller.audio.read_text(item.output)
                self.audio_play_id = item_id
            elif self.audio_play_id == item_id:
                if not self.window.controller.audio.is_playing():
                    self.window.controller.audio.read_text(item.output)
                else:
                    self.window.controller.audio.stop_output()
                    self.window.controller.audio.on_stop()
                    self.audio_play_id = None

    def join_item(self, item_id: int, force: bool = False):
        """
        Join ctx items

        :param item_id: Item id
        :param force: Force join
        """
        if not force:
            self.window.ui.dialogs.confirm(
                type='ctx.join_item',
                id=item_id,
                msg=trans('ctx.join.item.confirm'),
            )
            return
        prev_item = self.window.core.ctx.get_previous_item(item_id)
        current_item = self.window.core.ctx.get_item_by_id(item_id)
        if prev_item is not None and current_item is not None:
            prev_item.output += current_item.output
            self.window.core.ctx.update_item(prev_item)
            self.window.core.ctx.remove_item(current_item.id)
            self.window.controller.ctx.refresh()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.02.26 22:00:00                  #
# ================================================== #

from PySide6.QtWidgets import QDialog, QLabel, QVBoxLayout, QPushButton

from pygpt_net.ui.widget.element.labels import CmdLabel


class SnapDialogCamera(QDialog):
    def __init__(self, window=None):
        """
        Snap dialog for camera

        :param window: main window
        """
        super(SnapDialogCamera, self).__init__(window)
        self.window = window
        self.setParent(window)
        self.setWindowTitle("Snap detected")
        self.cmd = CmdLabel(self.window, "sudo snap connect pygpt:camera")

        self.btn = QPushButton("OK")
        self.btn.clicked.connect(self.accept)
        self.btn.setStyleSheet("margin: 10px 0px 0px 0px;")

        # layout
        self.layout = QVBoxLayout()
        self.message = QLabel(
            "Camera is not connected! It must be connected in Snap environment.\n"
            "Run the following command to enable the camera:")
        self.message.setStyleSheet("margin: 10px 0px 10px 0px;")
        self.layout.addWidget(self.message)
        self.layout.addWidget(self.cmd)
        self.layout.addWidget(self.btn)
        self.layout.addStretch()
        self.setLayout(self.layout)


class SnapDialogAudioInput(QDialog):
    def __init__(self, window=None):
        """
        Snap dialog for audio input

        :param window: main window
        """
        super(SnapDialogAudioInput, self).__init__(window)
        self.window = window
        self.setParent(window)
        self.setWindowTitle("Snap is detected")
        self.cmd = CmdLabel(self.window, "sudo snap connect pygpt:audio-record :audio-record")

        self.btn = QPushButton("OK")
        self.btn.clicked.connect(self.accept)
        self.btn.setStyleSheet("margin: 10px 0px 0px 0px;")

        # layout
        self.layout = QVBoxLayout()
        self.message = QLabel(
            "Microphone is not connected! It must be connected in Snap environment.\n"
            "Run the following command to enable the microphone:")
        self.message.setStyleSheet("margin: 10px 0px 10px 0px;")
        self.layout.addWidget(self.message)
        self.layout.addWidget(self.cmd)
        self.layout.addWidget(self.btn)
        self.layout.addStretch()
        self.setLayout(self.layout)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.03.11 01:00:00                  #
# ================================================== #

from .field.checkbox import Checkbox
from .field.combo import Combo
from .field.cmd import Cmd
from .field.dictionary import Dictionary
from .field.input import Input
from .field.slider import Slider
from .field.textarea import Textarea
from .placeholder import Placeholder


class Config:
    def __init__(self, window=None):
        """
        Configuration controller

        :param window: Window instance
        """
        self.window = window
        self.checkbox = Checkbox(window)
        self.combo = Combo(window)
        self.cmd = Cmd(window)
        self.dictionary = Dictionary(window)
        self.input = Input(window)
        self.placeholder = Placeholder(window)
        self.slider = Slider(window)
        self.textarea = Textarea(window)

    def load_options(self, parent_id: str, options: dict):
        """
        Load options

        :param parent_id: Parent ID
        :param options: Options dict
        """
        for key in options:
            option = options[key]
            self.apply(parent_id, key, option)

    def apply(self, parent_id: str, key: str, option: dict):
        """
        Apply option to field handler based on type

        :param parent_id: Parent ID
        :param key: Option key
        :param option: Option dict
        """
        if option['type'] == 'int' or option['type'] == 'float':
            if 'slider' in option and option['slider']:
                self.slider.apply(parent_id, key, option)
            else:
                self.input.apply(parent_id, key, option)
        elif option['type'] == 'text':
            self.input.apply(parent_id, key, option)
        elif option['type'] == 'textarea':
            self.textarea.apply(parent_id, key, option)
        elif option['type'] == 'bool':
            self.checkbox.apply(parent_id, key, option)
        elif option['type'] == 'dict':
            self.dictionary.apply(parent_id, key, option)
        elif option['type'] == 'combo':
            self.combo.apply(parent_id, key, option)
        elif option['type'] == 'cmd':
            self.cmd.apply(parent_id, key, option)

    def apply_value(self, parent_id: str, key: str, option: dict, value):
        """
        Apply value to option

        :param parent_id: Parent ID
        :param key: Option key
        :param option: Option dict
        :param value: Option value
        """
        option['value'] = value
        self.apply(parent_id, key, option)

    def get_value(self, parent_id: str, key: str, option: dict) -> any:
        """
        Get value from field handler based on type

        :param parent_id: Parent ID
        :param key: Option key
        :param option: Option dict
        :return: Option value
        """
        if option['type'] == 'int' or option['type'] == 'float':
            if 'slider' in option and option['slider']:
                return self.slider.get_value(parent_id, key, option)
            else:
                return self.input.get_value(parent_id, key, option)
        elif option['type'] == 'text':
            return self.input.get_value(parent_id, key, option)
        elif option['type'] == 'textarea':
            return self.textarea.get_value(parent_id, key, option)
        elif option['type'] == 'bool':
            return self.checkbox.get_value(parent_id, key, option)
        elif option['type'] == 'dict':
            return self.dictionary.get_value(parent_id, key, option)
        elif option['type'] == 'combo':
            return self.combo.get_value(parent_id, key, option)
        elif option['type'] == 'cmd':
            return self.cmd.get_value(parent_id, key, option)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.02.21 14:00:00                  #
# ================================================== #

import os
import shutil
from datetime import datetime
from pathlib import PurePath

from PySide6.QtCore import QUrl


class Filesystem:
    def __init__(self, window=None):
        """
        Filesystem core

        :param window: Window instance
        """
        self.window = window
        self.workdir_placeholder = "%workdir%"
        self.styles = [
            "style.css",
            "style.dark.css",
            "style.light.css",
            "markdown.css",
            "markdown.dark.css",
            "markdown.light.css",
            "fix_windows.css",
        ]

    def install(self):
        """Install provider data"""
        # data directory
        data_dir = self.window.core.config.get_user_dir('data')
        if not os.path.exists(data_dir):
            os.makedirs(data_dir, exist_ok=True)

        # upload directory
        upload_dir = self.window.core.config.get_user_dir('upload')
        if not os.path.exists(upload_dir):
            os.makedirs(upload_dir, exist_ok=True)

        # install custom css styles for override default styles
        css_dir = os.path.join(self.window.core.config.path, 'css')
        if not os.path.exists(css_dir):
            os.mkdir(css_dir)

        src_dir = os.path.join(self.window.core.config.get_app_path(), 'data', 'css')
        dst_dir = os.path.join(self.window.core.config.path, 'css')

        try:
            for style in self.styles:
                src = os.path.join(src_dir, style)
                dst = os.path.join(dst_dir, style)
                if not os.path.exists(dst) and os.path.exists(src):
                    shutil.copyfile(src, dst)
        except Exception as e:
            print("Error while installing css files: ", e)

    def make_local(self, path: str) -> str:
        """
        Make local placeholder path

        :param path: path to prepare
        :return: local path with working dir placeholder
        """
        return path.replace(
            self.window.core.config.get_user_path(),
            self.workdir_placeholder
        )

    def make_local_list(self, paths: list) -> list:
        """
        Make local placeholder paths

        :param paths: list with paths to prepare
        :return: local paths with working dir placeholder
        """
        return [self.make_local(path) for path in paths]

    def make_local_list_img(self, paths: list) -> list:
        """
        Make local placeholder paths for images

        :param paths: list with paths to prepare
        :return: local paths with working dir placeholder
        """
        img_ext = ['.png', '.jpg', '.jpeg', '.gif', '.bmp', '.tiff', '.webp']
        result = []
        for path in paths:
            if path.endswith(tuple(img_ext)):
                result.append(self.make_local(path))
        return result

    def get_url(self, url) -> QUrl:
        """
        Make current OS-specific URL to open file or directory

        :param url: URL to prepare
        :return: URL to open file or directory
        """
        if self.window.core.platforms.is_windows():
            if not url.startswith('file:///'):
                url = 'file:///' + url
            return QUrl(url, QUrl.TolerantMode)
        else:
            return QUrl.fromLocalFile(url)

    def get_path(self, path) -> str:
        """
        Prepare current OS-specific path from given path

        :param path: path to prepare
        :return: prepared OS-specific path
        """
        parts = PurePath(path).parts
        if len(parts) > 1:
            return os.path.join(*parts)  # rebuild OS directory separators
        return path

    def to_workdir(self, path) -> str:
        """
        Replace user path with current workdir

        :param path: path to fix
        :return: path with replaced user workdir
        """
        path = self.get_path(path)
        work_dir = self.window.core.config.get_user_path()  # current OS app workdir

        # try to find %workdir% placeholder in path
        if self.workdir_placeholder in path:
            return path.replace(
                self.workdir_placeholder,
                work_dir
            )

        # try to find workdir in path: old versions compatibility, < 2.0.113
        if work_dir.endswith('.config/pygpt-net'):
            work_dir = work_dir.rsplit('/.config/pygpt-net', 1)[0]
        elif work_dir.endswith('.config\\pygpt-net'):
            work_dir = work_dir.rsplit('\\.config\\pygpt-net', 1)[0]

        if self.window.core.platforms.is_windows():
            dir_index = path.find('\\.config\\pygpt-net\\') + 1
        else:
            dir_index = path.find('/.config/pygpt-net/') + 1

        parts = path[dir_index:]
        return os.path.join(work_dir, parts)

    def extract_local_url(self, path) -> (str, str):
        """
        Extract local url and path from url

        :param path: local path or url
        :return: url, path
        """
        if not self.is_schema(path):
            path = self.to_workdir(path)

        prefix = ''
        if not self.is_schema(path):
            if self.window.core.platforms.is_windows():
                prefix = 'file:///'
            else:
                prefix = 'file://'

        url = prefix + path
        return url, path

    def get_workdir_prefix(self) -> str:
        """
        Get workdir prefix

        :return: workdir prefix
        """
        prefix = ''
        if self.window.core.platforms.is_windows():
            prefix = 'file:///'
        else:
            prefix = 'file://'
        return prefix + self.window.core.config.get_user_path()

    def in_work_dir(self, path: str) -> bool:
        """
        Check if path is in work directory

        :param path: path to file
        :return: True if path is in work directory
        """
        work_dir = self.window.core.config.get_user_dir("data")
        return path.startswith(work_dir)

    def store_upload(self, path: str) -> str:
        """
        Store file in upload directory

        :param path: path to uploading file
        :return: path to stored uploaded file
        """
        upload_dir = self.window.core.config.get_user_dir("upload")
        file_name = os.path.basename(path)
        upload_path = os.path.join(upload_dir, file_name)
        # if file exists, add datetime prefix
        if os.path.exists(upload_path):
            upload_path = os.path.join(upload_dir, datetime.now().strftime("%Y-%m-%d_%H-%M-%S_") + file_name)
        shutil.copyfile(path, upload_path)
        return upload_path

    def remove_upload(self, path: str) -> None:
        """
        Delete uploaded file

        :param path: path to uploading file
        """
        upload_dir = self.window.core.config.get_user_dir("upload")
        if path.startswith(upload_dir):
            if os.path.exists(path):
                os.remove(path)

    def is_schema(self, path: str) -> bool:
        """
        Check if path has schema prefix (http, https, file)

        :param path: path to check
        :return: True if path has schema prefix
        """
        return path.startswith('file://') or path.startswith('http://') or path.startswith('https://')




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.02.25 22:00:00                  #
# ================================================== #


class UIDebug:
    def __init__(self, window=None):
        """
        UI debug

        :param window: Window instance
        """
        self.window = window
        self.id = 'ui'

    def map_structure(self, d, show_value: bool = False):
        """
        Map dict structure

        :param d: data to map
        :param show_value: show value
        :return: mapped structure
        """
        if isinstance(d, dict):
            return {k: self.map_structure(v, show_value) for k, v in d.items()}
        elif isinstance(d, list):
            return [self.map_structure(item, show_value) for item in d]
        else:
            if show_value:
                return d
            else:
                return type(d).__name__

    def update_section(self, items: dict, name: str):
        """
        Update debug items

        :param items: items
        :param name: name
        """
        self.window.core.debug.add(self.id, 'ui.' + name, str(self.map_structure(items)))

    def update(self):
        """Update debug window"""
        self.window.core.debug.begin(self.id)

        self.window.core.debug.add(
            self.id, '*lang_mapping',
            str(self.map_structure(self.window.controller.lang.mapping.get_mapping(), True))
        )

        self.update_section(self.window.ui.config, 'config')
        self.update_section(self.window.ui.debug, 'debug')
        self.update_section(self.window.ui.dialog, 'dialog')
        self.update_section(self.window.ui.editor, 'editor')
        self.update_section(self.window.ui.groups, 'groups')
        self.update_section(self.window.ui.hooks, 'hooks')
        self.update_section(self.window.ui.menu, 'menu')
        self.update_section(self.window.ui.models, 'models')
        self.update_section(self.window.ui.nodes, 'nodes')
        self.update_section(self.window.ui.notepad, 'notepad')
        self.update_section(self.window.ui.paths, 'paths')
        self.update_section(self.window.ui.plugin_addon, 'plugin_addon')
        self.update_section(self.window.ui.splitters, 'splitters')
        self.update_section(self.window.ui.tabs, 'tabs')

        self.window.core.debug.end(self.id)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.01.27 11:00:00                  #
# ================================================== #

from PySide6.QtGui import Qt
from PySide6.QtWidgets import QSplitter, QVBoxLayout, QWidget

from pygpt_net.ui.layout.toolbox.assistants import Assistants
from pygpt_net.ui.layout.toolbox.indexes import Indexes
from pygpt_net.ui.layout.toolbox.mode import Mode
from pygpt_net.ui.layout.toolbox.model import Model
from pygpt_net.ui.layout.toolbox.presets import Presets
from pygpt_net.ui.layout.toolbox.prompt import Prompt
from pygpt_net.ui.layout.toolbox.footer import Footer
from pygpt_net.ui.widget.element.labels import HelpLabel
from pygpt_net.utils import trans


class ToolboxMain:
    def __init__(self, window=None):
        """
        Toolbox UI

        :param window: Window instance
        """
        self.window = window
        self.assistants = Assistants(window)
        self.indexes = Indexes(window)
        self.footer = Footer(window)
        self.mode = Mode(window)
        self.model = Model(window)
        self.presets = Presets(window)
        self.prompt = Prompt(window)

    def setup(self) -> QSplitter:
        """
        Setup toolbox

        :return: QSplitter
        :rtype: QSplitter
        """
        # mode / model
        self.window.ui.splitters['toolbox.mode'] = QSplitter(Qt.Horizontal)
        self.window.ui.splitters['toolbox.mode'].addWidget(self.mode.setup())  # modes
        self.window.ui.splitters['toolbox.mode'].addWidget(self.model.setup())  # models

        # presets / assistants
        self.window.ui.splitters['toolbox.presets'] = QSplitter(Qt.Horizontal)
        self.window.ui.splitters['toolbox.presets'].addWidget(self.indexes.setup())  # indexes
        self.window.ui.splitters['toolbox.presets'].addWidget(self.presets.setup())  # prompts
        self.window.ui.splitters['toolbox.presets'].addWidget(self.assistants.setup())  # assistants

        bottom = QVBoxLayout()
        bottom.addWidget(self.prompt.setup())
        bottom.addWidget(self.footer.setup())

        bottom_widget = QWidget()
        bottom_widget.setLayout(bottom)

        self.window.ui.nodes['tip.toolbox.mode'] = HelpLabel(trans('tip.toolbox.mode'), self.window)
        self.window.ui.nodes['tip.toolbox.mode'].setAlignment(Qt.AlignCenter)

        layout_top = QVBoxLayout()
        layout_top.addWidget(self.window.ui.splitters['toolbox.mode'])
        layout_top.addWidget(self.window.ui.nodes['tip.toolbox.mode'])
        layout_top.setContentsMargins(0, 0, 0, 0)
        layout_top.setStretch(0, 1)

        widget_top = QWidget()
        widget_top.setLayout(layout_top)

        # rows
        self.window.ui.splitters['toolbox'] = QSplitter(Qt.Vertical)
        self.window.ui.splitters['toolbox'].addWidget(widget_top)  # mode/model
        self.window.ui.splitters['toolbox'].addWidget(self.window.ui.splitters['toolbox.presets'])  # presets/assists.
        self.window.ui.splitters['toolbox'].addWidget(bottom_widget)  # system prompt, footer (names, temp, logo, etc.)

        return self.window.ui.splitters['toolbox']



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.03.06 22:00:00                  #
# ================================================== #

from pygpt_net.plugin.base import BasePlugin
from pygpt_net.core.dispatcher import Event
from pygpt_net.item.ctx import CtxItem


class Plugin(BasePlugin):
    def __init__(self, *args, **kwargs):
        super(Plugin, self).__init__(*args, **kwargs)
        self.id = "agent"
        self.name = "Autonomous Agent (inline)"
        self.description = "Enables inline autonomous mode (Agent) in current mode. " \
                           "WARNING: Please use with caution - this mode, when connected with other plugins, " \
                           "may produce unexpected results!"
        self.type = [
            "agent",
        ]
        self.order = 9998
        self.use_locale = True
        self.init_options()

    def init_options(self):
        """
        Initialize options
        """
        prompt = "AUTONOMOUS MODE:\n1. You will now enter self-dialogue mode, where you will be conversing with " \
                 "yourself, not with a human.\n2. When you enter self-dialogue mode, remember that you are engaging " \
                 "in a conversation with yourself. Any user input will be considered a reply featuring your " \
                 "previous response.\n3. The objective of this self-conversation is well-defined—focus " \
                 "on achieving it.\n4. Your new message should be a continuation of the last response you generated, " \
                 "essentially replying" \
                 " to yourself and extending it.\n5. After each response, critically evaluate its effectiveness " \
                 "and alignment with the goal. If necessary, refine your approach.\n6. Incorporate self-critique " \
                 "after every response to capitalize on your strengths and address areas needing improvement.\n7. To " \
                 "advance towards the goal, utilize all the strategic thinking and resources at your disposal.\n" \
                 "8. Ensure that the dialogue remains coherent and logical, with each response serving as a stepping " \
                 "stone towards the ultimate objective.\n" \
                 "9. Treat the entire dialogue as a monologue aimed at devising" \
                 " the best possible solution to the problem.\n10. Conclude the self-dialogue upon realizing the " \
                 "goal or reaching a pivotal conclusion that meets the initial criteria.\n11. You are allowed to use " \
                 "any commands and tools without asking for it.\n12. While using commands, always use the correct " \
                 "syntax and never interrupt the command before generating the full instruction.\n13. ALWAYS break " \
                 "down the main task into manageable logical subtasks, systematically addressing and analyzing each" \
                 " one in sequence.\n14. With each subsequent response, make an effort to enhance your previous " \
                 "reply by enriching it with new ideas and do it automatically without asking for it.\n15. Any input " \
                 "that begins with 'user: ' will come from me, and I will be able to provide you with ANY additional " \
                 "commands or goal updates in this manner. " \
                 "The other inputs, not prefixed with 'user: ' will represent" \
                 " your previous responses.\n16. Start by breaking down the task into as many smaller sub-tasks as " \
                 "possible, then proceed to complete each one in sequence.  Next, break down each sub-task into even " \
                 "smaller tasks, carefully and step by step go through all of them until the required goal is fully " \
                 "and correctly achieved.\n"
        extended_prompt = "AUTONOMOUS MODE:\n1. You will now enter self-dialogue mode, where you will be conversing " \
                          "with yourself, not with a human.\n2. When you enter self-dialogue mode, remember that you " \
                          "are engaging in a conversation with yourself. Any user input will be considered a reply " \
                          "featuring your previous response.\n3. The objective of this self-conversation is " \
                          "well-defined—focus on achieving it.\n4. Your new message should be a continuation of " \
                          "the last response you generated, essentially replying to yourself and extending it.\n" \
                          "5. After each response, critically evaluate its effectiveness and alignment with the goal." \
                          " If necessary, refine your approach.\n6. Incorporate self-critique after every response " \
                          "to capitalize on your strengths and address areas needing improvement.\n7. To advance " \
                          "towards the goal, utilize all the strategic thinking and resources at your disposal.\n" \
                          "8. Ensure that the dialogue remains coherent and logical, with each response serving " \
                          "as a stepping stone towards the ultimate objective.\n9. Treat the entire dialogue " \
                          "as a monologue aimed at devising the best possible solution to the problem.\n" \
                          "10. Conclude the self-dialogue upon realizing the goal or reaching a pivotal conclusion " \
                          "that meets the initial criteria.\n11. You are allowed to use any commands and tools " \
                          "without asking for it.\n12. While using commands, always use the correct syntax " \
                          "and never interrupt the command before generating the full instruction.\n13. Break down " \
                          "the main task into manageable logical subtasks, systematically addressing and analyzing " \
                          "each one in sequence.\n14. With each subsequent response, make an effort to enhance " \
                          "your previous reply by enriching it with new ideas and do it automatically without " \
                          "asking for it.\n15. Any input that begins with 'user: ' will come from me, and I will " \
                          "be able to provide you with ANY additional commands or goal updates in this manner. " \
                          "The other inputs, not prefixed with 'user: ' will represent your previous responses.\n" \
                          "16. Start by breaking down the task into as many smaller sub-tasks as possible, then " \
                          "proceed to complete each one in sequence.  Next, break down each sub-task into even " \
                          "smaller tasks, carefully and step by step go through all of them until the required " \
                          "goal is fully and correctly achieved.\n17. Always split every step into parts: " \
                          "main goal, current sub-task, potential problems, pros and cons, criticism of the " \
                          "previous step, very detailed (about 10-15 paragraphs) response to current subtask, " \
                          "possible improvements, next sub-task to achieve and summary.\n18. Do not start the " \
                          "next subtask until you have completed the previous one.\n19. Ensure to address and " \
                          "correct any criticisms or mistakes from the previous step before starting the next " \
                          "subtask.\n20. Do not finish until all sub-tasks and the main goal are fully achieved " \
                          "in the best possible way. If possible, improve the path to the goal until the full " \
                          "objective is achieved.\n21. Conduct the entire discussion in my native language.\n" \
                          "22. Upon reaching the final goal, provide a comprehensive summary including " \
                          "all solutions found, along with a complete, expanded response."
        self.add_option(
            "iterations",
            type="int",
            value=3,
            label="Iterations",
            description="How many iterations to run? 0 = infinite.\n"
                        "WARNING: setting this to 0 can cause a lot of requests and heavy tokens usage!",
            min=0,
            max=100,
            multiplier=1,
            step=1,
            slider=True,
        )
        # prompts list
        keys = {
            "enabled": "bool",
            "name": "text",
            "prompt": "textarea",
        }
        items = [
            {
                "enabled": True,
                "name": "Default",
                "prompt": prompt,
            },
            {
                "enabled": False,
                "name": "Extended",
                "prompt": extended_prompt,
            },
        ]
        desc = "Prompt used to instruct how to handle autonomous mode, you can create as many prompts as you want." \
               "First active prompt on list will be used to handle autonomous mode."
        tooltip = desc
        self.add_option(
            "prompts",
            type="dict",
            value=items,
            label="Prompts",
            description=desc,
            tooltip=tooltip,
            keys=keys,
        )
        self.add_option(
            "auto_stop",
            type="bool",
            value=True,
            label="Auto-stop after goal is reached",
            description="If enabled, plugin will stop after goal is reached.",
        )
        self.add_option(
            "reverse_roles",
            type="bool",
            value=True,
            label="Reverse roles between iterations",
            description="If enabled, roles will be reversed between iterations.",
        )

    def setup(self) -> dict:
        """
        Return available config options

        :return: config options
        """
        return self.options

    def attach(self, window):
        """
        Attach window

        :param window: Window instance
        """
        self.window = window

    def is_allowed(self) -> bool:
        """
        Check if plugin is allowed to run

        :return: True if plugin is allowed to run
        """
        return self.window.core.config.get("mode") != "agent"  # check global mode

    def handle(self, event: Event, *args, **kwargs):
        """
        Handle dispatched event

        :param event: event object
        :param args: event args
        :param kwargs: event kwargs
        """
        name = event.name
        data = event.data
        ctx = event.ctx

        always_events = [Event.FORCE_STOP, Event.PLUGIN_SETTINGS_CHANGED, Event.ENABLE, Event.DISABLE]

        if not self.is_allowed() and name != Event.DISABLE:
            return

        if name not in always_events and not self.is_active_prompt():
            return

        if name == Event.CTX_BEFORE:
            self.on_ctx_before(ctx)

        elif name == Event.CTX_AFTER:
            self.on_ctx_after(ctx)

        elif name == Event.CTX_END:
            self.on_ctx_end(ctx)

        elif name == Event.USER_SEND:
            self.on_user_send(data['value'])

        elif name == Event.FORCE_STOP:
            self.on_stop()

        elif name == Event.SYSTEM_PROMPT:
            data['value'] = self.on_system_prompt(data['value'])

        elif name == Event.INPUT_BEFORE:
            data['value'] = self.on_input_before(data['value'])

        elif name in [
            Event.ENABLE,
            Event.DISABLE,
        ]:
            if data['value'] == self.id:
                self.window.controller.agent.update()  # update agent status bar

        elif name == Event.PLUGIN_SETTINGS_CHANGED:
            self.window.controller.agent.update()  # update agent status bar

        elif name in [
            Event.CMD_INLINE,
            Event.CMD_EXECUTE,
        ]:
            if self.get_option_value("auto_stop"):
                self.cmd(
                    ctx,
                    data['commands'],
                )

    def is_active_prompt(self) -> bool:
        """
        Check if active prompt is enabled

        :return: True if active prompt is enabled
        """
        for item in self.get_option_value("prompts"):
            if item["enabled"]:
                return True
        return False

    def on_system_prompt(self, prompt: str) -> str:
        """
        Event: SYSTEM_PROMPT

        :param prompt: prompt
        :return: updated prompt
        """
        pre_prompt = ("YOU ARE NOW AN AUTONOMOUS AGENT AND YOU ARE ENTERING NOW INTO AGENT MODE.\n"
                      "Use below instructions in every agent run iteration:\n\n")
        return pre_prompt + self.window.controller.agent.flow.on_system_prompt(
            prompt,
            append_prompt=self.get_first_active_prompt(),
            auto_stop=self.get_option_value("auto_stop"),
        )

    def on_input_before(self, prompt: str) -> str:
        """
        Event: INPUT_BEFORE

        :param prompt: prompt
        :return: updated prompt
        """
        return self.window.controller.agent.flow.on_input_before(prompt)

    def cmd(self, ctx: CtxItem, cmds: list):
        """
        Events: CMD_INLINE, CMD_EXECUTE

        :param ctx: CtxItem
        :param cmds: commands dict
        """
        self.window.controller.agent.flow.cmd(ctx, cmds)  # force execute

    def on_stop(self):
        """
        Event: FORCE_STOP
        """
        self.window.controller.agent.flow.on_stop()  # force stop

    def on_user_send(self, text: str):
        """
        Event: USER_SEND

        :param text: text
        """
        self.window.controller.agent.flow.on_user_send(text)

    def on_ctx_end(self, ctx: CtxItem):
        """
        Event: CTX_END

        :param ctx: CtxItem
        """
        self.window.controller.agent.flow.on_ctx_end(
            ctx,
            iterations=int(self.get_option_value("iterations")),
        )

    def on_ctx_before(self, ctx: CtxItem):
        """
        Event: CTX_BEFORE

        :param ctx: CtxItem
        """
        self.window.controller.agent.flow.on_ctx_before(
            ctx,
            reverse_roles=self.get_option_value("reverse_roles"),
        )

    def on_ctx_after(self, ctx: CtxItem):
        """
        Event: CTX_AFTER

        :param ctx: CtxItem
        """
        self.window.controller.agent.flow.on_ctx_after(ctx)

    def get_first_active_prompt(self) -> str:
        """
        Get first active prompt from prompts list

        :return: system prompt
        """
        for item in self.get_option_value("prompts"):
            if item["enabled"]:
                return item["prompt"]
        return ""



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.01.26 18:00:00                  #
# ================================================== #

from pygpt_net.item.ctx import CtxItem
from .chat import Chat
from .completion import Completion


class Chain:
    def __init__(self, window=None):
        """
        Langchain wrapper core

        :param window: Window instance
        """
        self.window = window
        self.chat = Chat(window)
        self.completion = Completion(window)

    def call(self, **kwargs) -> bool:
        """
        Call LLM using Langchain

        :param kwargs: keyword arguments
        :return: result
        """
        prompt = kwargs.get("prompt", "")
        system_prompt = kwargs.get("system_prompt", "")
        stream = kwargs.get("stream", False)
        model = kwargs.get("model", None)
        ctx = kwargs.get("ctx", CtxItem())
        user_name = ctx.input_name  # from ctx
        ai_name = ctx.output_name  # from ctx
        response = None
        used_tokens = 0
        sub_mode = 'chat'

        # get available sub-modes
        if 'mode' in model.langchain:
            if 'chat' in model.langchain['mode']:
                sub_mode = 'chat'
            elif 'completion' in model.langchain['mode']:
                sub_mode = 'completion'

        try:
            if sub_mode == 'chat':
                response = self.chat.send(
                    prompt=prompt,
                    system_prompt=system_prompt,
                    ai_name=ai_name,
                    user_name=user_name,
                    stream=stream,
                    model=model,
                )
                used_tokens = self.chat.get_used_tokens()
            elif sub_mode == 'completion':
                response = self.completion.send(
                    prompt=prompt,
                    system_prompt=system_prompt,
                    ai_name=ai_name,
                    user_name=user_name,
                    stream=stream,
                    model=model,
                )
                used_tokens = self.completion.get_used_tokens()

        except Exception as e:
            self.window.core.debug.log(e)
            raise e  # re-raise to window

        # if stream mode, store stream
        if stream:
            ctx.stream = response
            ctx.input_tokens = used_tokens  # get from input tokens calculation
            ctx.set_output("", ai_name)
            return True

        if response is None:
            return False

        # get output
        output = None
        if sub_mode == 'chat':
            output = response.content
        elif sub_mode == 'completion':
            output = response

        # store context
        ctx.set_output(output, ai_name)

        return True



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.02.29 01:00:00                  #
# ================================================== #

import os

from PySide6.QtCore import QSize
from PySide6.QtGui import Qt, QIcon
from PySide6.QtWidgets import QVBoxLayout, QLabel, QHBoxLayout, QPushButton, QWidget, QSizePolicy

from .agent import Agent
from .image import Image
from .indexes import Indexes
from .vision import Vision
from pygpt_net.ui.widget.textarea.name import NameInput
from pygpt_net.ui.widget.option.slider import OptionSlider
from pygpt_net.utils import trans


class Footer:
    def __init__(self, window=None):
        """
        Toolbox UI

        :param window: Window instance
        """
        self.window = window
        self.agent = Agent(window)
        self.image = Image(window)
        self.indexes = Indexes(window)
        self.vision = Vision(window)

    def setup(self) -> QWidget:
        """
        Setup footer

        :return: QHBoxLayout
        """
        # AI and users names
        # names_layout = QHBoxLayout()
        # names_layout.addLayout(self.setup_name_input('preset.ai_name', trans("toolbox.name.ai")))
        # names_layout.addLayout(self.setup_name_input('preset.user_name', trans("toolbox.name.user")))

        # bottom
        option = dict(self.window.controller.settings.editor.get_options()["temperature"])
        self.window.ui.nodes['temperature.label'] = QLabel(trans("toolbox.temperature.label"))
        self.window.ui.config['global']['current_temperature'] = \
            OptionSlider(self.window, 'global', 'current_temperature', option)
        self.window.ui.add_hook("update.global.current_temperature", self.window.controller.mode.hook_global_temperature)

        # per mode options
        rows = QVBoxLayout()
        # rows.addWidget(self.window.ui.nodes['temperature.label'])
        # rows.addWidget(self.window.ui.config['global']['current_temperature'])
        rows.addWidget(self.agent.setup())
        rows.addWidget(self.image.setup())
        # rows.addWidget(self.vision.setup())
        rows.addWidget(self.indexes.setup_options())
        rows.setContentsMargins(0, 0, 0, 0)

        # logo
        logo_button = self.setup_logo()

        # bottom (options and logo)
        # bottom = QHBoxLayout()
        # bottom.addLayout(rows, 80)
        # bottom.addWidget(logo_button, 20)
        # bottom.setStretchFactor(logo_button, 1)
        # bottom.setAlignment(logo_button, Qt.AlignRight | Qt.AlignBottom)
        # bottom_widget = QWidget()
        # bottom_widget.setLayout(bottom)

        # layout rows
        # layout = QVBoxLayout()
        # layout.addLayout(names_layout)
        # layout.addWidget(bottom_widget)

        widget = QWidget()
        widget.setLayout(rows)

        return widget

    def setup_name_input(self, id: str, title: str) -> QVBoxLayout:
        """
        Setup name input

        :param id: ID of the input
        :param title: Title of the input
        :return: QVBoxLayout
        """
        label_key = 'toolbox.' + id + '.label'
        self.window.ui.nodes[label_key] = QLabel(title)
        self.window.ui.nodes[id] = NameInput(self.window, id)

        layout = QVBoxLayout()
        layout.addWidget(self.window.ui.nodes[label_key])
        layout.addWidget(self.window.ui.nodes[id])

        return layout

    def setup_logo(self) -> QPushButton:
        """
        Setup logo

        :return: QPushButton
        """
        path = os.path.abspath(os.path.join(self.window.core.config.get_app_path(), 'data', 'logo.png'))

        button = QPushButton()
        button.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Expanding)
        button.setIcon(QIcon(path))
        button.setIconSize(QSize(100, 28))
        button.setFlat(True)
        button.clicked.connect(lambda: self.window.controller.dialogs.info.goto_website())

        return button



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.02.21 14:00:00                  #
# ================================================== #

from PySide6.QtCore import QObject, Signal, QRunnable, Slot

from pygpt_net.item.assistant import AssistantItem


class Importer:
    def __init__(self, window=None):
        """
        Importer core

        :param window: Window instance
        """
        self.window = window

    @Slot(str, object)
    def handle_error(self, mode: str, err: any):
        """
        Handle thread error signal

        :param mode: mode
        :param err: error message
        """
        if mode == "assistants":
            self.window.controller.assistant.handle_imported_assistants_failed(err)
        elif mode == "files":
            self.window.controller.assistant.files.handle_imported_files_failed(err)

    @Slot(str, int)
    def handle_finished(self, mode: str, num: int):
        """
        Handle thread finished signal

        :param mode: mode
        :param num: number of imported items
        """
        if mode == "assistants":
            self.window.controller.assistant.handle_imported_assistants(num)
        elif mode == "files":
            self.window.controller.assistant.files.handle_imported_files(num)

    def import_assistants(self):
        """Import assistants"""
        worker = ImportWorker()
        worker.window = self.window
        worker.mode = "assistants"
        worker.signals.finished.connect(self.handle_finished)
        worker.signals.error.connect(self.handle_error)
        self.window.threadpool.start(worker)

    def import_files(self, assistant: AssistantItem):
        """
        Import assistant files

        :param assistant: assistant
        """
        worker = ImportWorker()
        worker.window = self.window
        worker.mode = "files"
        worker.assistant = assistant
        worker.signals.finished.connect(self.handle_finished)
        worker.signals.error.connect(self.handle_error)
        self.window.threadpool.start(worker)


class ImportWorkerSignals(QObject):
    finished = Signal(str, int)
    error = Signal(str, object)


class ImportWorker(QRunnable):
    def __init__(self, *args, **kwargs):
        super(ImportWorker, self).__init__()
        self.signals = ImportWorkerSignals()
        self.window = None
        self.mode = "assistants"
        self.assistant = None

    @Slot()
    def run(self):
        """Importer thread"""
        try:
            # import data
            if self.mode == "assistants":
                self.import_assistants()
            elif self.mode == "files":
                self.import_files(self.assistant)

        except Exception as e:
            self.signals.error.emit(self.mode, e)

    def import_assistants(self, silent: bool = False) -> bool:
        """
        Import assistants from API

        :param silent: silent mode (no signals)
        :return: result
        """
        try:
            items = self.window.core.assistants.get_all()
            self.window.core.gpt.assistants.import_api(items)
            self.window.core.assistants.items = items
            self.window.core.assistants.save()

            # import uploaded files
            for id in self.window.core.assistants.items:
                assistant = self.window.core.assistants.get_by_id(id)
                self.import_files(assistant, True)

            if not silent:
                self.signals.finished.emit("assistants", len(items))
            return True
        except Exception as e:
            self.signals.error.emit("assistants", e)
            return False

    def import_files(self, assistant: AssistantItem, silent: bool = False) -> bool:
        """
        Import assistant files from API

        :param assistant: assistant
        :param silent: silent mode (no signals)
        :return: result
        """
        try:
            files = self.window.core.gpt.assistants.file_list(assistant.id)
            self.window.core.assistants.import_files(assistant, files)
            self.window.core.assistants.save()

            if not silent:
                self.signals.finished.emit("files", len(files))
            return True
        except Exception as e:
            self.signals.error.emit("files", e)
        return False
